require "image_tinify/version"
require 'chunky_png'
require 'tinify'

require_relative 'command'
require_relative 'key_manager'

module ImageTinify

  # 做标记的 Key
  MARKUP_KEY = 'IMG_TINIFY'
  
  # 做标记的 Value
  MARKUP_VALUE = '1'

  # 目标扩展名列表
  TARGET_EXT = ['.png']

  class ImageTinify

    attr_accessor :root_only

    def initialize(tiny_png_key = nil)
      @validate = false
      Tinify.key = tiny_png_key
    end

    def validate?
      @validate
    end

    def validate!
      @validate = Tinify.validate!
    end

    def relative_path(path)
      path.gsub(@root_dir, '.')
    end

    # 查找所有图片
    def _find_all_images(path, block)
      if File.directory? path
        return if @root_only && path != @root_dir
        Dir.foreach(path)
          .reject { |x| x.start_with? '.' }
          .each { |name| _find_all_images File.join(path, name), block }
      else
        block.call path if TARGET_EXT.include? File.extname(path)
      end
    end

    def find_all_images(path, block)
      @root_dir = path
      _find_all_images(path, block)
    end
  
    # 查找所有未压缩的图片
    def find_uncompressed_images(images)
      images.each { |file|
        begin
          stream = ChunkyPNG::Datastream.from_file(file)
  
          compressed = stream.other_chunks
            .select { |ck| ck.type.downcase == 'itxt' }
            .reduce(false) { |m, ck|
              m || (ck.keyword == MARKUP_KEY && ck.text == MARKUP_VALUE)
            }
  
          yield file if block_given? && !compressed
          
        rescue => e
          puts e.to_s
        end
      }
    end

    # 压缩图片
    def compress_image(path)
      validate! unless validate?
      Tinify.from_file(path).to_file(path)
      true
    end

    # 标记已压缩
    def mark_compressed(path)
      stream = ChunkyPNG::Datastream.from_file(path)
      stream.other_chunks << ChunkyPNG::Chunk::InternationalText.new(MARKUP_KEY, MARKUP_VALUE)
      stream.save(path)
      true
    end
  
  end
end
